@@ -5,18 +5,20 @@
 """
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, division, print_function
 
-from builtins import str
+
 
 # Ensure setting pyglet.options['debug_gl'] to False is done prior to any
 # other calls to pyglet or pyglet submodules, otherwise it may not get picked
 # up by the pyglet GL engine and have no effect.
 # Shaders will work but require OpenGL2.0 drivers AND PyOpenGL3.0+
 import pyglet
+
+from ..colors import Color
+
 pyglet.options['debug_gl'] = False
 import ctypes
 GL = pyglet.gl
@@ -87,15 +89,11 @@ class RadialStim(GratingStim):
         self._initParams = dir()
         self._initParams.remove('self')
 
-        super(RadialStim, self).__init__(win, units=units, name=name,
+        super(RadialStim, self).__init__(win, units=units, name=name, size=size,
                                          autoLog=False)  # start off false
 
-        # use shaders if available by default, this is a good thing
-        self.useShaders = win._haveShaders
-
         # UGLY HACK again. (See same section in GratingStim for ideas)
         self.__dict__['contrast'] = 1
-        self.__dict__['size'] = 1
         self.__dict__['sf'] = 1
         self.__dict__['tex'] = tex
 
@@ -117,18 +115,18 @@ class RadialStim(GratingStim):
         self.colorSpace = colorSpace
         if rgb != None:
             logging.warning("Use of rgb arguments to stimuli are deprecated."
-                            " Please use color and colorSpace args instead")
-            self.setColor(rgb, colorSpace='rgb', log=False)
+                            " Please use color and colorSpace args instead.")
+            self.color = Color(rgb, space='rgb')
         elif dkl != None:
-            logging.warning("Use of dkl arguments to stimuli are deprecated. "
-                            "Please use color and colorSpace args instead")
-            self.setColor(dkl, colorSpace='dkl', log=False)
+            logging.warning("Use of dkl arguments to stimuli are deprecated."
+                            " Please use color and colorSpace args instead.")
+            self.color = Color(dkl, space='dkl')
         elif lms != None:
             logging.warning("Use of lms arguments to stimuli are deprecated."
-                            " Please use color and colorSpace args instead")
-            self.setColor(lms, colorSpace='lms', log=False)
+                            " Please use color and colorSpace args instead.")
+            self.color = Color(lms, space='lms')
         else:
-            self.setColor(color, log=False)
+            self.color = color
 
         self.ori = float(ori)
         self.__dict__['angularRes'] = angularRes
@@ -140,7 +138,7 @@ class RadialStim(GratingStim):
         self.pos = numpy.array(pos, float)
         self.depth = depth
         self.__dict__['sf'] = 1
-        self.size = val2array(size, False)
+        self.size = size
 
         self.tex = tex
         self.mask = mask
@@ -177,11 +175,11 @@ class RadialStim(GratingStim):
         res = self.texRes  # resolution of texture - 128 is bearable
         step = 1.0/res
         rad = numpy.arange(0, 1 + step, step)
-        if type(self.mask) == numpy.ndarray:
+        if isinstance(self.mask, numpy.ndarray):
             # handle a numpy array
             intensity = 255 * self.mask.astype(float)
             res = len(intensity)
-        elif type(self.mask) == list:
+        elif isinstance(self.mask, list):
             # handle a numpy array
             intensity = 255 * numpy.array(self.mask, float)
             res = len(intensity)
@@ -215,10 +213,9 @@ class RadialStim(GratingStim):
             res = im.size[0]
             im = im.convert("L")  # force to intensity (in case it was rgb)
             intensity = numpy.asarray(im)
-            fromFile = 1
 
         data = intensity.astype(numpy.uint8)
-        mask = data.tostring()  # serialise
+        mask = data.tobytes()  # serialise
 
         # do the openGL binding
         if self.interpolate:
@@ -339,15 +336,10 @@ class RadialStim(GratingStim):
         edge2 = (self._angles + self._triangleWidth) * (180/pi) > visW[1]
         self._visible[edge2] = False
         self._nVisible = numpy.sum(self._visible) * 3
-
         self._updateTextureCoords()
         self._updateMaskCoords()
         self._updateVerticesBase()
         self._updateVertices()  # is this necessary? Works fine without...
-        if not self.useShaders:
-            # generate a displaylist ID
-            self._listID = GL.glGenLists(1)
-            self._updateList()  # ie refresh display list
 
     @attributeSetter
     def angularRes(self, value):
@@ -385,69 +377,58 @@ class RadialStim(GratingStim):
         GL.glPushMatrix()  # push before the list, pop after
         # scale the viewport to the appropriate size
         self.win.setScale('pix')
-        if self.useShaders:
-            # setup color
-            desiredRGB = self._getDesiredRGB(self.rgb, self.colorSpace,
-                                             self.contrast)
-            GL.glColor4f(desiredRGB[0], desiredRGB[1], desiredRGB[2],
-                         self.opacity)
+        # setup color
+        GL.glColor4f(*self._foreColor.render('rgba1'))
 
-            # assign vertex array
-            GL.glVertexPointer(2, GL.GL_DOUBLE, 0, self.verticesPix.ctypes)
+        # assign vertex array
+        GL.glVertexPointer(2, GL.GL_DOUBLE, 0, self.verticesPix.ctypes)
 
-            # then bind main texture
-            GL.glActiveTexture(GL.GL_TEXTURE0)
-            GL.glBindTexture(GL.GL_TEXTURE_2D, self._texID)
-            GL.glEnable(GL.GL_TEXTURE_2D)
-            # and mask
-            GL.glActiveTexture(GL.GL_TEXTURE1)
-            GL.glBindTexture(GL.GL_TEXTURE_1D, self._maskID)
-            GL.glDisable(GL.GL_TEXTURE_2D)
-            GL.glEnable(GL.GL_TEXTURE_1D)
+        # then bind main texture
+        GL.glActiveTexture(GL.GL_TEXTURE0)
+        GL.glBindTexture(GL.GL_TEXTURE_2D, self._texID)
+        GL.glEnable(GL.GL_TEXTURE_2D)
+        # and mask
+        GL.glActiveTexture(GL.GL_TEXTURE1)
+        GL.glBindTexture(GL.GL_TEXTURE_1D, self._maskID)
+        GL.glDisable(GL.GL_TEXTURE_2D)
+        GL.glEnable(GL.GL_TEXTURE_1D)
 
-            # setup the shaderprogram
-            prog = self.win._progSignedTexMask1D
-            GL.glUseProgram(prog)
-            # set the texture to be texture unit 0
-            GL.glUniform1i(GL.glGetUniformLocation(prog, b"texture"), 0)
-            # mask is texture unit 1
-            GL.glUniform1i(GL.glGetUniformLocation(prog, b"mask"), 1)
+        # setup the shaderprogram
+        prog = self.win._progSignedTexMask1D
+        GL.glUseProgram(prog)
+        # set the texture to be texture unit 0
+        GL.glUniform1i(GL.glGetUniformLocation(prog, b"texture"), 0)
+        # mask is texture unit 1
+        GL.glUniform1i(GL.glGetUniformLocation(prog, b"mask"), 1)
 
-            # set pointers to visible textures
-            GL.glClientActiveTexture(GL.GL_TEXTURE0)
-            GL.glTexCoordPointer(2, GL.GL_DOUBLE, 0,
-                                 self._visibleTexture.ctypes)
-            GL.glEnableClientState(GL.GL_TEXTURE_COORD_ARRAY)
+        # set pointers to visible textures
+        GL.glClientActiveTexture(GL.GL_TEXTURE0)
+        GL.glTexCoordPointer(2, GL.GL_DOUBLE, 0,
+                             self._visibleTexture.ctypes)
+        GL.glEnableClientState(GL.GL_TEXTURE_COORD_ARRAY)
 
-            # mask
-            GL.glClientActiveTexture(GL.GL_TEXTURE1)
-            GL.glTexCoordPointer(1, GL.GL_DOUBLE, 0,
-                                 self._visibleMask.ctypes)
-            GL.glEnableClientState(GL.GL_TEXTURE_COORD_ARRAY)
+        # mask
+        GL.glClientActiveTexture(GL.GL_TEXTURE1)
+        GL.glTexCoordPointer(1, GL.GL_DOUBLE, 0,
+                             self._visibleMask.ctypes)
+        GL.glEnableClientState(GL.GL_TEXTURE_COORD_ARRAY)
 
-            # do the drawing
-            GL.glEnableClientState(GL.GL_VERTEX_ARRAY)
-            GL.glDrawArrays(GL.GL_TRIANGLES, 0, self._nVisible)
+        # do the drawing
+        GL.glEnableClientState(GL.GL_VERTEX_ARRAY)
+        GL.glDrawArrays(GL.GL_TRIANGLES, 0, self._nVisible)
 
-            # unbind the textures
-            GL.glClientActiveTexture(GL.GL_TEXTURE1)
-            GL.glBindTexture(GL.GL_TEXTURE_2D, 0)
-            # main texture
-            GL.glClientActiveTexture(GL.GL_TEXTURE0)
-            GL.glBindTexture(GL.GL_TEXTURE_2D, 0)
-            GL.glDisable(GL.GL_TEXTURE_2D)
-            # disable set states
-            GL.glDisableClientState(GL.GL_VERTEX_ARRAY)
-            GL.glDisableClientState(GL.GL_TEXTURE_COORD_ARRAY)
+        # unbind the textures
+        GL.glClientActiveTexture(GL.GL_TEXTURE1)
+        GL.glBindTexture(GL.GL_TEXTURE_2D, 0)
+        # main texture
+        GL.glClientActiveTexture(GL.GL_TEXTURE0)
+        GL.glBindTexture(GL.GL_TEXTURE_2D, 0)
+        GL.glDisable(GL.GL_TEXTURE_2D)
+        # disable set states
+        GL.glDisableClientState(GL.GL_VERTEX_ARRAY)
+        GL.glDisableClientState(GL.GL_TEXTURE_COORD_ARRAY)
 
-            GL.glUseProgram(0)
-        else:
-            # the list does the texture mapping
-            if self._needTextureUpdate:
-                self.setTex(value=self.tex, log=False)
-            if self._needUpdate:
-                self._updateList()
-            GL.glCallList(self._listID)
+        GL.glUseProgram(0)
 
         # return the view to previous state
         GL.glPopMatrix()
@@ -471,6 +452,7 @@ class RadialStim(GratingStim):
         vertsBase /= 2.0  # size should be 1.0, so radius should be 0.5
         vertsBase = vertsBase[self._visible, :, :]
         self._verticesBase = vertsBase.reshape(self._nVisible, 2)
+        self.vertices = self._verticesBase
 
     def _updateTextureCoords(self):
         """calculate texture coordinates if angularCycles or Phase change
@@ -568,61 +550,10 @@ class RadialStim(GratingStim):
         # setup the shaderprogram
         GL.glEndList()
 
-    def _updateListNoShaders(self):
-        """The user shouldn't need this method since it gets called
-        after every call to .set() Basically it updates the OpenGL
-        representation of your stimulus if some parameter of the
-        stimulus changes. Call it if you change a property manually
-        rather than using the .set() command
-        """
-        self._needUpdate = False
-        GL.glNewList(self._listID, GL.GL_COMPILE)
-        # glColor can interfere with multitextures
-        GL.glColor4f(1.0, 1.0, 1.0, self.opacity)
-
-        # assign vertex array
-        GL.glVertexPointer(2, GL.GL_DOUBLE, 0, self.verticesPix.ctypes)
-        GL.glEnableClientState(GL.GL_VERTEX_ARRAY)
-
-        # bind and enable textures
-        # main texture
-        GL.glActiveTexture(GL.GL_TEXTURE0)
-        GL.glBindTexture(GL.GL_TEXTURE_2D, self._texID)
-        GL.glEnable(GL.GL_TEXTURE_2D)
-        # mask
-        GL.glActiveTexture(GL.GL_TEXTURE1)
-        GL.glBindTexture(GL.GL_TEXTURE_1D, self._maskID)
-        GL.glDisable(GL.GL_TEXTURE_2D)
-        GL.glEnable(GL.GL_TEXTURE_1D)
-
-        # set pointers to visible textures
-        # mask
-        GL.glClientActiveTexture(GL.GL_TEXTURE1)
-        GL.glTexCoordPointer(2, GL.GL_DOUBLE, 0, self._visibleMask.ctypes)
-        GL.glEnableClientState(GL.GL_TEXTURE_COORD_ARRAY)
-        # texture
-        GL.glClientActiveTexture(GL.GL_TEXTURE0)
-        GL.glTexCoordPointer(2, GL.GL_DOUBLE, 0, self._visibleTexture.ctypes)
-        GL.glEnableClientState(GL.GL_TEXTURE_COORD_ARRAY)
-
-        # do the drawing
-        GL.glDrawArrays(GL.GL_TRIANGLES, 0, self._nVisible)
-
-        # disable set states
-        GL.glDisableClientState(GL.GL_VERTEX_ARRAY)
-        GL.glActiveTexture(GL.GL_TEXTURE0)
-        GL.glDisableClientState(GL.GL_TEXTURE_COORD_ARRAY)
-        GL.glActiveTexture(GL.GL_TEXTURE1)
-        GL.glDisableClientState(GL.GL_TEXTURE_COORD_ARRAY)
-
-        GL.glEndList()
-
     def __del__(self):
         """Remove textures from graphics card to prevent crash
         """
         try:
-            if not self.useShaders:
-                GL.glDeleteLists(self._listID, 1)
             self.clearTextures()
-        except ModuleNotFoundError:
+        except (ImportError, ModuleNotFoundError, TypeError):
             pass  # has probably been garbage-collected already
